package net.sf.itcb.addons.monitoring.track;

import net.sf.itcb.addons.monitoring.track.appender.ItcbAppender;
import net.sf.itcb.addons.monitoring.track.bean.ItcbTrackBean;
import net.sf.itcb.common.business.core.ItcbConstants;

import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.Signature;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.slf4j.MDC;
import org.springframework.beans.factory.annotation.Required;

/**
 * This interceptor's aim is to log time spent in each layer and the result {@link TracerInterceptor#Status}.<br/>
 * It defines a transaction UUID if not already defined for the current thread.<br/>
 * This id will allow the monitoring visual tool to show the spent time in each layer.<br/>
 * <br/>
 * This interceptor has to be declared in Spring context file as in the example bellow :
 * <pre>{@code 
   	<aop:config>
		<aop:aspect id="aopItcbAddonTrackerInterceptorWithArgsOnTechnicalErrors"  ref="itcbAddonTrackerInterceptorWithArgsOnlyOnTechnicalErrors">
			    <aop:around
			      pointcut-ref="server" 
			      method="track"/>
		</aop:aspect>
		<aop:aspect id="aopItcbAddonTrackerInterceptorWithNoArgs"  ref="itcbAddonTrackerInterceptorWithNoArgsOnErrors">
			    <aop:around
			      pointcut-ref="page" 
			      method="track"/>
			    <aop:around
			      pointcut-ref="service" 
			      method="track"/>
			    <aop:around
			      pointcut-ref="dao" 
			      method="track"/>
		</aop:aspect>
	</aop:config>
 
 }</pre>
 <br/>
 * You can use and adapt this declaration in order to track specifically each layer. Available values for <code>ref</code> are :
 * <ul>
 * <li>itcbAddonTrackerInterceptorWithArgsOnAllErrors</li>
 * <li>itcbAddonTrackerInterceptorWithArgsOnlyOnTechnicalErrors</li>
 * <li>itcbAddonTrackerInterceptorWithArgsOnlyOnBusinessErrors</li>
 * <li>itcbAddonTrackerInterceptorWithNoArgsOnErrors</li>
 * </ul>
 * 
 * You just have to take care, not to use twice a pattern. Tracking would be done twice. So the list must contain only one interceptor.<br/>
 * 
 * You also can add your own pointcut. Basic pointcut are already declared for page, server, service (client and server side) and dao. <br/>
 * You can see it in itcb-monitoring-trace-applicationContext.xml in this package.<br/>
 * 
 * @author Pierre Le Roux
 * @since ITCB common 0.2.0
 * 
 */

public class TrackerInterceptor {

	private ItcbAppender itcbAppender;
	
	@Required
	public void setItcbAppender(ItcbAppender itcbAppender) {
		this.itcbAppender = itcbAppender;
	}

	protected final Logger logger = LoggerFactory.getLogger(getClass());
	
	public Object track(ProceedingJoinPoint pjp) throws Throwable {
		 long startTime = System.currentTimeMillis();
		  
		 ItcbTrackBean itcbTrackBean = new ItcbTrackBean();
		 itcbTrackBean.setStartTime(startTime);
		 itcbTrackBean.setTransactionId(TransactionIdHolder.getTransactionId());
		 try {
	    	Object ret=pjp.proceed();
	    	return ret;
	     }
	     catch(Throwable t) {
		    itcbTrackBean.setException(t);
	    	throw t;
	     }
	     finally {
	    	Signature signature =pjp.getSignature(); 
	    	itcbTrackBean.setTimeInMillis(System.currentTimeMillis() - startTime);
	    	itcbTrackBean.setClassName(signature.getDeclaringTypeName());
	    	itcbTrackBean.setMethod(signature.getName());
	    	itcbTrackBean.setUser(MDC.get(ItcbConstants.MDC_USER_KEY));
	    	itcbTrackBean.setParameters(pjp.getArgs());
	    	itcbAppender.track(itcbTrackBean);
	     }

	 }	
	
}