package net.sf.itcb.addons.monitoring.track.server;

import java.io.StringWriter;

import javax.xml.transform.OutputKeys;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import net.sf.itcb.addons.monitoring.track.Globals;
import net.sf.itcb.addons.monitoring.track.TransactionIdHolder;
import net.sf.itcb.addons.monitoring.track.appender.ItcbAppender;
import net.sf.itcb.addons.monitoring.track.bean.ItcbTrackBean;
import net.sf.itcb.common.business.core.ItcbConstants;

import org.slf4j.MDC;
import org.springframework.beans.factory.annotation.Required;
import org.springframework.ws.context.MessageContext;
import org.springframework.ws.server.EndpointInterceptor;
import org.springframework.ws.soap.SoapHeader;
import org.springframework.ws.soap.SoapMessage;

/**
 * This server interceptor receives the transaction id from the client.<br/>
 * It retrieves it in SoapHeader with {@link Globals#HEADER_TRANSACTION_ID_QNAME} Qname.
 * 
 * This interceptor has to be activated in your server modules spring configuration as in the following example :<br/>
 * <code>
 *   &lt;bean class="org.springframework.ws.server.endpoint.mapping.PayloadRootAnnotationMethodEndpointMapping"&gt;<br/>
 *	    &lt;property name="interceptors"&gt;<br/>
 *		    &lt;list&gt;<br/>
 *            &lt;ref bean="trackerEndpointInterceptor"/&gt;<br/>
 *          ...<br/>
 *		      &lt;ref bean="validatingInterceptor" /&gt;<br/>
 *          ...<br/>
 *		      
 *		    &lt;/list&gt;<br/>
 *		&lt;/property&gt;<br/>
 *   &lt;/bean&gt;<br/><br/>
 *   
 *   &lt;bean id="trackerEndpointInterceptor" class="net.sf.itcb.addons.monitoring.track.server.TrackerEndpointInterceptor"&gt;
 *   	&lt;property name="itcbAppender" ref="xxxxYourAppender"/&gt;
 *   &lt;/bean&gt;
 * </code> 
 * <br/>
 * This declaration has to be done in the webservice config file in your src/main/webapp/WEB-INF/xxxx-servlet.xml
 * @author Pierre Le Roux
 *
 */
public class TrackerEndpointInterceptor implements EndpointInterceptor {

	private static final String PROPERTY_TIME_START =  "net.sf.itcb.addons.monitoring.track.server.TrackerNoTrackEndpointInterceptor";
	
	private static final TransformerFactory transformerFactory = TransformerFactory.newInstance();
	
	private ItcbAppender itcbAppender;
	
	@Required
	public void setItcbAppender(ItcbAppender itcbAppender) {
		this.itcbAppender = itcbAppender;
	}
	
	@Override
	public boolean handleFault(MessageContext messageContext, Object endpoint)
			throws Exception {
		handle(messageContext, endpoint, true);
		return true;
	}

	@Override
	public boolean handleRequest(MessageContext messageContext, Object endpoint)
			throws Exception {
		SoapMessage soapMessage = ((SoapMessage) messageContext.getRequest());
		SoapHeader soapHeader = soapMessage.getSoapHeader();
		String transactionId=null;
		if( soapHeader != null) {
			transactionId = soapHeader.getAttributeValue(Globals.HEADER_TRANSACTION_ID_QNAME);
			
			// If transactionId has been set, we store it in the current Thread	
			if(transactionId!=null) {
				TransactionIdHolder.setTransactionId(transactionId);
			}
			else {
				// else we initialize it
				transactionId = TransactionIdHolder.getTransactionId();
			}
		}
		else {
			// else we initialize it
			transactionId= TransactionIdHolder.getTransactionId();
		}
		messageContext.setProperty(PROPERTY_TIME_START, System.currentTimeMillis());
		
		return true;
	}

	@Override
	public boolean handleResponse(MessageContext messageContext, Object endpoint)
			throws Exception {
		handle(messageContext, endpoint, false);
		return true;
	}
	
	
	private void handle(MessageContext messageContext, Object endpoint, boolean isFault) {
		 ItcbTrackBean itcbTrackBean = new ItcbTrackBean();
		 itcbTrackBean.setClassName(getClass().getName());
		 SoapMessage soapRequest = ((SoapMessage) messageContext.getRequest());
		 //récupération de l'opération invoquée
         if(soapRequest != null) {
	        Source source = soapRequest.getPayloadSource();
	
	        if (source != null && source instanceof DOMSource) {
	            DOMSource dom = (DOMSource) source;
	            org.w3c.dom.Node node = dom.getNode();
	            itcbTrackBean.setMethod(node.getLocalName());
	            itcbTrackBean.setStartTime((Long) messageContext.getProperty(PROPERTY_TIME_START));
	            itcbTrackBean.setTimeInMillis(System.currentTimeMillis() - (Long) messageContext.getProperty(PROPERTY_TIME_START));
	            itcbTrackBean.setTransactionId(TransactionIdHolder.getTransactionId());
	            if(isFault) {
	            	
	            	SoapMessage soapResponse = ((SoapMessage) messageContext.getResponse());
	            	itcbTrackBean.setException(new SoapRequestToString(soapResponse));
	            	itcbTrackBean.setParameters(new Object[] {new SoapRequestToString(soapRequest)});
	            }
	            itcbTrackBean.setUser(MDC.get(ItcbConstants.MDC_USER_KEY));
	            itcbAppender.track(itcbTrackBean);
	        }
         }
 		TransactionIdHolder.closeTransaction();
		
	}

	
	/**
	 * An inner class which aim is to hold the soapRequest in order to be printed in asynchronous method track
	 * Extends exception only 
	 * @author Pierre Le Roux
	 * 
	 */
	public class SoapRequestToString extends Exception{

		private static final long serialVersionUID = 1L;
		private SoapMessage soapMessage;
		
		public SoapRequestToString (SoapMessage soapMessage) {
			this.soapMessage = soapMessage;
		}
		
		@Override
		public String toString() {
			Transformer transformer;
			try {
				transformer = transformerFactory.newTransformer();

				transformer.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");
				transformer.setOutputProperty(OutputKeys.INDENT, "no");

				StringWriter writer = new StringWriter();
				transformer.transform(soapMessage.getEnvelope().getSource(), new StreamResult(writer));
				return writer.toString();
			} catch (Exception e) {
				e.printStackTrace();
				return "Error in soapmessage toString";
			}
			 
		}

		@Override
		public String getMessage() {
			return toString();
		}
	}
}
